      subroutine  readmm5(iunit,num3d,num2d,num1d,num0d,ifield,
     &  ifieldskip,numpar2d,numpar3d,err,eof)

* READS MM5 OUTPUT, LOOKS WETHER IT IS NEEDED BY FLEXPART,
* DECOUPLES THE OUTPUT FIELDS, INTERPOLATES THEM TO DOT POINTS
* (MM5 DOES NOT USE ARAKAWA-C GRID)
*
* INPUT:
*
* iunit            fortran unit of mm5 output
* num3d            number of 3d parameters
* num2d            number of 2d parameters
* num1d            number of 1d parameters
* num0d            number of 0d parameters
* ifield           field number since start of simulation
* ifieldskip       number of fields not used for FLEXPART (initialization)
* numpar2d         number of 2D fields needed for FLEXPART
* numpar3d         number of 3D fields needed for FLEXPART
*
* OUTPUT:
*
* err              occurance of read error (.true./.false.)
* eof              reached end of file (.true./.false.)

      implicit none
      include 'inclpar'
      include 'inclcom'
      integer iunit,num3d,num2d,num1d,num0d,ifield,ifieldskip
      integer numpar2d,numpar3d,i,kk,ii,jj,idot,ipar,ixh,jxh
      logical err,eof
      REAL DUM3D(IX,JX,KX), DUM3D2(IX,JX,KX,MAX3DMM5),
     *     DUM2D(IX,JX),    DUM2D2(IX,JX),
     *     DUM1D(KX), DUM0D,
     *     PSTAR(IX,JX),  PSTARD(IX,JX)
      real dum2dd(ix,jx)
      character id*8

      ixh=ix/2
      jxh=jx/2
      if(ifield.eq.1) then
        do 5 ii=1,ix
          do 5 jj=1,jx
            do 5 kk=1,max2dmm5
5             valbefore(ii,jj,kk)=0.0
      endif

      err=.false.
      eof=.false.

* First, read header...
      READ(IUNIT,END=999,ERR=888) MIF,MRF,MIFC,MRFC

* Second, read the 3D parameters
      DO 120 I=1,NUM3D
         IDOT=1-MIF(204+I,MIF(1,1))/10
         ID=MIFC(204+I,MIF(1,1))(1:8)
         IF(ID.EQ.'W       ' .AND. MIF(  10,1).EQ.2) THEN
              READ(IUNIT) DUM2D,DUM3D
              do kk = 1,kx
              do ii = 1,ix
              do jj = 1,jx
                 if (kk.eq.1) dum2d2(ii,jj) = dum2d(ii,jj)
                 dum3d2(ii,jj,kk,i) = dum3d(ii,jj,kk)
              end do
              end do
              end do
         ELSE
              READ (IUNIT) DUM3D
              do kk = 1,kx
              do ii = 1,ix
              do jj = 1,jx
                 dum3d2(ii,jj,kk,i) = dum3d(ii,jj,kk)
              end do
              end do
              end do
         ENDIF
 120  CONTINUE

* Third, read the 2D parameters
      DO 140 I=1,NUM2D
         IDOT=1-MIF(204+NUM3D+I,MIF(1,1))/10
         ID=MIFC(204+NUM3D+I,MIF(1,1))(1:8)
         READ (IUNIT) DUM2D

         IF (ID.EQ.'PSTARCRS') THEN
           do ii = 1,ix
             do jj = 1,jx
               pstar(ii,jj) = dum2d(ii,jj)
             end do
           end do
         END IF

* CHECK WHETHER THIS IS A NEEDED 2D PARAMETER OR NOT
         if(ifield.gt.ifieldskip) then
           do 125 ipar=1,numpar2d
             if(id.eq.mm5name2d(ipar)(1:8)) then
               if(cdot2d(ipar).eq.'C') then
                 CALL CRS2DOT(DUM2D,DUM2DD,IX,JX)
                 do 123 ii=1,ix
                   do 123 jj=1,jx
                     vmm5_2d(ipar,ii,jj)=(dum2dd(ii,jj)-
     &                 valbefore(ii,jj,i))*rfac2d(ipar)
                     if(mm5name2d(ipar)(1:8).eq.'PSTARCRS') then
                       vmm5_2d(ipar,ii,jj)=vmm5_2d(ipar,ii,jj)+ptop
                     endif
                     if(cacc2d(ipar).eq.'Y') then
                       valbefore(ii,jj,i)=dum2dd(ii,jj)
                     else
                       valbefore(ii,jj,i)=0.0
                     endif
123              continue
               else 
                 do 124 ii=1,ix
                   do 124 jj=1,jx
                     vmm5_2d(ipar,ii,jj)=(dum2d(ii,jj)-
     &                 valbefore(ii,jj,i))*rfac2d(ipar)
                     if(cacc2d(ipar).eq.'Y') then
                       valbefore(ii,jj,i)=dum2d(ii,jj)
                     else
                       valbefore(ii,jj,i)=0.0
                     endif
124              continue
               endif
               write(*,'(a,2x,e8.2)')
     &           '      EXTRACTED 2D PARAMETER: '//mm5name2d(ipar),
     &           vmm5_2d(ipar,ixh,jxh)
             endif
125        continue
         endif

140   CONTINUE

      DO 160 I=1,NUM1D
         READ (IUNIT) DUM1D
160   CONTINUE

      DO 180 I=1,NUM0D
         READ (IUNIT) DUM0D
180   CONTINUE

* CALCULATE DOT-POINT PSTAR FROM CROSS-POINT PSTAR

      CALL CRS2DOT(PSTAR,PSTARD,IX,JX)

* DECOUPLE COUPLED MM5 FIELDS

      DO 200 I=1,NUM3D
         IDOT=1-MIF(204+I,MIF(1,1))/10
         ID=MIFC(204+I,MIF(1,1))(1:8)

         IF(ID.NE.'RAD TEND' .AND. ID.NE.'TKE     ') THEN
            DO KK = 1,KX
            DO II = 1,IX-IDOT
            DO JJ = 1,JX-IDOT
              IF (ID.EQ.'U       '.OR.ID.EQ.'V       ') THEN
                DUM3D2(II,JJ,KK,I) = DUM3D2(II,JJ,KK,I)/PSTARD(II,JJ)
              ELSE
                DUM3D2(II,JJ,KK,I) = DUM3D2(II,JJ,KK,I)/PSTAR(II,JJ)
                IF (ID.EQ.'W       '.AND.KK.EQ.1) THEN
                  DUM2D2(II,JJ) = DUM2D2(II,JJ)/PSTAR(II,JJ)
                END IF
              END IF
            END DO
            END DO
            END DO
         END IF
200   CONTINUE


      do 300 i=1,num3d
         ID=MIFC(204+I,MIF(1,1))(1:8)

* CHECK WHETHER THIS IS A NEEDED 3D PARAMETER OR NOT
         if(ifield.gt.ifieldskip) then
           do 225 ipar=1,numpar3d
             if(id.eq.mm5name3d(ipar)(1:8)) then
               do 224 kk=1,kx
                 IF (ID.EQ.'W       '.AND.KK.EQ.1) THEN
                   CALL CRS2DOT(DUM2D2,DUM2DD,IX,JX)
                   do 219 ii=1,ix
                     do 219 jj=1,jx
219                    wwtop(ii,jj)=dum2dd(ii,jj)*
     &                 rfac3d(ipar)
                 ENDIF
                 if(cdot3d(ipar).eq.'C') then
                   do 223 ii=1,ix
                     do 223 jj=1,jx
223                    dum2d(ii,jj)=dum3d2(II,JJ,KK,I)
                   CALL CRS2DOT(DUM2D,DUM2DD,IX,JX)
                   do 222 ii=1,ix
                     do 222 jj=1,jx
                       vmm5_3d(ipar,ii,jj,kk)=dum2dd(ii,jj)*
     &                 rfac3d(ipar)
222                continue
                 else 
                   do 221 ii=1,ix
                     do 221 jj=1,jx
                       vmm5_3d(ipar,ii,jj,kk)=dum3d2(II,JJ,KK,I)*
     &                 rfac3d(ipar)
221                continue
                 endif
224            continue
               write(*,'(a,2x,5(e8.2,1x))')
     &           '      EXTRACTED 3D PARAMETER: '//mm5name3d(ipar),
     &           (vmm5_3d(ipar,ixh,jxh,kk),kk=kx,1,-5)
             endif
225        continue
         endif
300   continue

      return

888   err=.true.
      return

999   eof=.true.
      return

      end
